@@ -23,9 +23,8 @@ class PDeinterlacer:
     The clip will need to be loaded from PSourcer to work as it needs it's Props.
     """
 
-    def __init__(self, clip, tff=True, kernel=None, kernel_args=None, debug=False):
+    def __init__(self, clip, kernel=None, kernel_args=None, debug=False):
         self.clip = clip
-        self.tff = tff
         self.kernel = kernel
         self.kernel_args = kernel_args
         self.debug = debug
@@ -44,7 +43,7 @@ class PDeinterlacer:
                     "Preset": "Placebo",
                     "MatchPreset": "Placebo",
                     "MatchPreset2": "Placebo",
-                    "TFF": self.tff,
+                    "TFF": True,  # handle this automatically
                     "InputType": 0,
                     "SourceMatch": 3,
                     "Lossless": 2,
@@ -57,24 +56,38 @@ class PDeinterlacer:
                 # user configuration
                 **dict(self.kernel_args or {})
             }
-        self.props = self.clip.get_frame(0).props
-        self.props = {k: v.decode("utf-8") if type(v) == bytes else v for k, v in self.props.items()}
-        if self.props["PVSSourcer"] == "core.d2v.Source":
-            self._d2v()
-        elif self.props["PVSSourcer"] == "core.ffms2.Source":
-            if kernel == havsfunc.QTGMC:
+        # set handler func based on Sourcer
+        self.handler = None
+        sourcer = self.clip.get_frame(0).props["PVSSourcer"].decode("utf-8")
+        if sourcer == "core.d2v.Source":
+            self.handler = self._d2v
+        elif sourcer == "core.ffms2.Source":
+            if self.kernel == havsfunc.QTGMC:
                 kernel_args["FPSDivisor"] = 2  # only supporting same-rate fps atm
-            self._ffms2()
-        elif self.props["PVSSourcer"] == "core.lsmas.LWLibavSource":
-            if kernel == havsfunc.QTGMC:
+            self.handler = self._ffms2
+        elif sourcer == "core.lsmas.LWLibavSource":
+            if self.kernel == havsfunc.QTGMC:
                 kernel_args["FPSDivisor"] = 2  # only supporting same-rate fps atm
-            self._lsmash()
-        elif self.props["PVSSourcer"] == "core.imwri.Read":
+            self.handler = self._lsmash
+        elif sourcer == "core.imwri.Read":
             print("pvsfunc.PDeinterlacer: Warning: This source is a clip of images and cannot be deinterlaced.")
+            self.handler = lambda c: c  # do nothing
         else:
-            raise ValueError(f"Unimplemented deinterlacer for Sourcer {self.props['PVSSourcer']}")
+            raise ValueError(f"Unimplemented deinterlacer for Sourcer {sourcer}")
+        self.clip = self.handler(self.clip)
     
-    def _d2v(self):
+    def _get_kernel(self, clip):
+        """
+        Apply the deinterlacing kernel to the provided clip for both
+        TFF and BFF output. The Kernel function will be provided a 
+        True/False value to the "TFF" argument.
+        """
+        return (
+            self.kernel(clip, **{**self.kernel_args, "TFF": True}),
+            self.kernel(clip, **{**self.kernel_args, "TFF": False})
+        )
+
+    def _d2v(self, clip):
         """
         Very accurate deinterlacing using raw frame metadata to know what to
         deinterlace when necessary. It even fixes the frame rates of progressive
@@ -83,16 +96,16 @@ class PDeinterlacer:
         For MPEG2, this is as good as it gets in terms of using a deinterlacer.
         """
         # 1. create a clip from the output of the kernel deinterlacer
-        deinterlaced_clip = self.kernel(self.clip, **self.kernel_args)
-        fps_factor = deinterlaced_clip.fps.numerator / deinterlaced_clip.fps.denominator
-        fps_factor = fps_factor / (self.clip.fps.numerator / self.clip.fps.denominator)
+        deinterlaced_tff, deinterlaced_bff = self._get_kernel(clip)
+        fps_factor = deinterlaced_tff.fps.numerator / deinterlaced_tff.fps.denominator
+        fps_factor = fps_factor / (clip.fps.numerator / clip.fps.denominator)
         if fps_factor != 1.0 and fps_factor != 2.0:
             raise ValueError(
-                f"pvsfunc.PDeinterlacer: The deinterlacer kernel returned an unsupported frame-rate ({deinterlaced_clip.fps}). "
+                f"pvsfunc.PDeinterlacer: The deinterlacer kernel returned an unsupported frame-rate ({deinterlaced_tff.fps}). "
                 "Only single-rate and double-rate is supported with PDeinterlacer at the moment."
             )
         # 2. deinterlace whats interlaced
-        def _d(n, f, c, d, ff):
+        def _d(n, f, c, d_tff, d_bff, ff):
             if f.props["PVSFlagProgressiveFrame"]:
                 # progressive frame, we don't need to do any deinterlacing to this frame
                 # though we may need to duplicate it if double-rate fps output
@@ -102,43 +115,50 @@ class PDeinterlacer:
                     f" VOB: {f.props['PVSFlagVob']}:{f.props['PVSFlagCell']} - Frame #{n:,} - Untouched ",
                     alignment=1
                 ) if self.debug else rc
-            # interlaced frame, we need to use `d` (deinterlaced) frame.
+            # interlaced frame, we need to use `d_tff`/`d_bff` (deinterlaced) frame.
             return core.text.Text(
-                d,
+                d_tff if f.props["_FieldBased"] == 2 else d_bff,
                 f" VOB: {f.props['PVSFlagVob']}:{f.props['PVSFlagCell']} - Frame #{n:,} - Deinterlaced! ",
                 alignment=1
-            ) if self.debug else d
-        self.clip = core.std.FrameEval(
-            deinterlaced_clip,
+            ) if self.debug else (d_tff if f.props["_FieldBased"] == 2 else d_bff)
+        return core.std.FrameEval(
+            deinterlaced_tff,
             functools.partial(
                 _d,
-                c=self.clip,
-                d=deinterlaced_clip,
+                c=clip,
+                d_tff=deinterlaced_tff,
+                d_bff=deinterlaced_bff,
                 ff=fps_factor
             ),
-            prop_src=self.clip
+            prop_src=clip
         )
     
-    def _ffms2(self):
+    def _ffms2(self, clip):
         """
         Deinterlace using ffms2 (ffmpeg) using a basic FieldBased!=0 => QTGMC method
         """
-        self.clip = core.std.FrameEval(
-            self.clip,
+        deinterlaced_tff, deinterlaced_bff = self._get_kernel(clip)
+        return core.std.FrameEval(
+            clip,
             functools.partial(
-                lambda n, f, c, d: (
+                lambda n, f, c, d_tff, d_bff: (
                     core.text.Text(c, "Untouched Frame (_FieldBased=0)", alignment=1) if self.debug else c
                 ) if f.props["_FieldBased"] == 0 else (
-                    core.text.Text(d, "Deinterlaced Frame (via QTGMC)", alignment=1) if self.debug else d
+                    core.text.Text(
+                        d_tff if f.props["_FieldBased"] == 2 else d_bff,
+                        "Deinterlaced Frame (via QTGMC)",
+                        alignment=1
+                    ) if self.debug else (d_tff if f.props["_FieldBased"] == 2 else d_bff)
                 ),
-                c=self.clip,
-                d=self.kernel(self.clip, **self.kernel_args)
+                c=clip,
+                d_tff=deinterlaced_tff,
+                d_bff=deinterlaced_bff
             ),
-            prop_src=self.clip
+            prop_src=clip
         )
     
-    def _lsmash(self):
+    def _lsmash(self, clip):
         """
         Deinterlace using lsmas (lsmash) using a basic FieldBased!=0 => QTGMC method
         """
-        self._ffms2()  # same method as ffms2
+        return self._ffms2(clip)  # same method as ffms2
